@@ -81,9 +81,7 @@ absl::Status ValidateDescriptor(const Descriptor& descriptor) {
 
     bool has_legacy_default = false;
     for (const auto& d : field.options().edition_defaults()) {
-      if (d.edition() == Edition::EDITION_LEGACY ||
-          // TODO Remove this once all features use EDITION_LEGACY.
-          d.edition() == Edition::EDITION_PROTO2) {
+      if (d.edition() == Edition::EDITION_LEGACY) {
         has_legacy_default = true;
         continue;
       }
@@ -189,17 +187,33 @@ absl::Status ValidateExtension(const Descriptor& feature_set,
   return absl::OkStatus();
 }
 
+void MaybeInsertEdition(Edition edition, Edition maximum_edition,
+                        absl::btree_set<Edition>& editions) {
+  if (edition <= maximum_edition) {
+    editions.insert(edition);
+  }
+}
+
+// This collects all of the editions that are relevant to any features defined
+// in a message descriptor.  We only need to consider editions where something
+// has changed.
 void CollectEditions(const Descriptor& descriptor, Edition maximum_edition,
                      absl::btree_set<Edition>& editions) {
   for (int i = 0; i < descriptor.field_count(); ++i) {
-    for (const auto& def : descriptor.field(i)->options().edition_defaults()) {
-      if (maximum_edition < def.edition()) continue;
-      // TODO Remove this once all features use EDITION_LEGACY.
-      if (def.edition() == Edition::EDITION_LEGACY) {
-        editions.insert(Edition::EDITION_PROTO2);
-        continue;
-      }
-      editions.insert(def.edition());
+    const FieldOptions& options = descriptor.field(i)->options();
+    // Editions where a new feature is introduced should be captured.
+    MaybeInsertEdition(options.feature_support().edition_introduced(),
+                       maximum_edition, editions);
+
+    // Editions where a feature is removed should be captured.
+    if (options.feature_support().has_edition_removed()) {
+      MaybeInsertEdition(options.feature_support().edition_removed(),
+                         maximum_edition, editions);
+    }
+
+    // Any edition where a default value changes should be captured.
+    for (const auto& def : options.edition_defaults()) {
+      MaybeInsertEdition(def.edition(), maximum_edition, editions);
     }
   }
 }
@@ -363,8 +377,10 @@ absl::StatusOr<FeatureSetDefaults> FeatureResolver::CompileDefaults(
   }
   // Sanity check validation conditions above.
   ABSL_CHECK(!editions.empty());
-  // TODO Check that this is always EDITION_LEGACY.
-  ABSL_CHECK_LE(*editions.begin(), EDITION_PROTO2);
+  if (*editions.begin() != EDITION_LEGACY) {
+    return Error("Minimum edition ", *editions.begin(),
+                 " is not EDITION_LEGACY");
+  }
 
   if (*editions.begin() > minimum_edition) {
     return Error("Minimum edition ", minimum_edition,
