@@ -58,7 +58,6 @@
 #include "google/protobuf/io/zero_copy_stream_impl.h"
 #include "google/protobuf/test_util.h"
 #include "google/protobuf/test_util2.h"
-#include "google/protobuf/unittest.pb.h"
 
 #if HAVE_ZLIB
 #include "google/protobuf/io/gzip_stream.h"
@@ -1756,3 +1755,5 @@ TEST(ZeroSizeArray, Output) {
 }  // namespace io
 }  // namespace protobuf
 }  // namespace google
+
+#include "google/protobuf/port_undef.inc"
